# -*-python-*-

__package__    = "modestMMarkers/modestMMarkers/__init__.py"
__version__    = "0.1"
__author__     = "Aaron Straup Cope"
__url__        = "http://www.aaronland.info/python/modestMMarkers"
__date__       = "$Date: 2009/04/24 16:08:49 $"
__copyright__  = "Copyright (c) 2009 Aaron Straup Cope. BSD license : http://www.modestmaps.com/license.txt"

import ModestMaps
import cairo
import PIL.Image

import math
import copy
import random
import array

from modestMMarkers import primitives
from modestMMarkers import utils

class modestMMarkers :

    def __init__ (self, mm_obj, rows=1, cols=1) :
        self.mm_obj = mm_obj
        self.markers = []
        self.legend = []

        self.rows = rows
        self.cols = cols

        self.sect_width = 0
        self.sect_height = 0
        
    # ##########################################################

    def plot_markers (self, mm_img, markers, **kwargs) :

        (w, h) = mm_img.size

        self.sect_width = int(w / self.cols)
        self.sect_height = int(h / self.rows)

        map_surface = utils.pil2cairo(mm_img)
        map_markers = []
        
        bg_c = (.5, 0, 0)
        str_c = (1, 1, 1)
        txt_c = (1, 1, 1)

        if kwargs.has_key('background_c'):
            bg_c = kwargs['background_c']

        if kwargs.has_key('stroke_c'):
            str_c = kwargs['stroke_c']

        if kwargs.has_key('text_c'):
            txt_c = kwargs['text_c']

        sorted = self.reposition_markers(markers)
        
        i = 1

        for data in sorted :

            # FIX ME: rename and make less confusing because these
            # have nothing to do with, for example, the rows and columns
            # for the legend on a turkishmmap

            data['row'] = int(math.ceil(float(data['y']) / float(self.sect_height)))
            data['col'] = int(math.ceil(float(data['x']) / float(self.sect_width)))
            data['id'] = i
            
            self.legend.append(data)

            args = {
                'x' : data['x'],
                'y' : data['y'],
                'radius' : 10,
                'string' : i,
                'background_c':bg_c,
                'stroke_c':str_c,
                'text_c' : txt_c,
                'font' : 'Helvetica-bold',
                'anchor_pos':data['anchor_pos'], 'anchor_height':data['anchor_height'],
                }

            primitives.draw_circle_with_string(map_surface, args)

            i += 1
            
        return (utils.cairo2pil(map_surface), self.legend)

    # ##########################################################
    
    def reposition_markers (self, markers) :

        for item in markers :
            data = copy.copy(item)
            data['height'] = 30
            data['width'] = 30
            data['anchor_pos'] = 'bottom'
            data['anchor_height'] = 0
            self.markers.append(data)

        self.sort_markers()
        self.__reposition_markers()

        return self.markers

    # ##########################################################
    
    def sort_markers (self):

        def mysort (x, y) :
            return cmp (y['latitude'], x['latitude'])

        self.markers.sort(mysort)

    # ##########################################################
    
    def reload_markers (self) :

        # it's called reload marker because it simply
        # calculates all the coordinates for a marker
        # but does not draw it
        
        for mrk_data in self.markers :
            
            w = mrk_data['width']
            h = mrk_data['height']
            a = mrk_data['anchor_height']

            hw = int(w / 2)
            hh = int(h / 2)
            
            loc = ModestMaps.Geo.Location(mrk_data['latitude'], mrk_data['longitude'])
            pt = self.mm_obj.locationPoint(loc)            

            x = int(pt.x)
            y = int(pt.y)
            
            mrk_data['x'] = x
            mrk_data['y'] = y

            # remember this is only the "canvas" - it should
            # not include the extra space taken up by the height
            # of the anchor...
            
            if mrk_data['anchor_pos'] == 'left' :
                pass
            
            elif mrk_data['anchor_pos'] == 'top' :
                pass
            
            elif mrk_data['anchor_pos'] == 'right' :
                pass

            else :
                
                x1 = x - hw
                y1 = y - (a + h)
                
                x2 = x + hw                
                y2 = y - a
                
            mrk_data['canvas'] = (x1, y1, x2, y2)

    # ##########################################################
    
    def __reposition_markers (self, iterations=1, max_iterations=100) :

        # get some context
        self.reload_markers()
        
        # the number of markers
        count = len(self.markers)

        # markers, accounting for 0-based index
        indexes = range(0, count)

        # start from the bottom
        indexes.reverse()

        # happy happy
        try_again = False

        for offset_mrk in range(0, count) : 

            mrk_idx = indexes[offset_mrk]
            current = self.markers[mrk_idx]

            next = offset_mrk + 1

            for offset_test in range(next, count) :

                test_idx = indexes[offset_test]
                other = self.markers[test_idx]

                overlap = self.does_marker_overlap_marker(current, other)
                
                if overlap != 0 :
                    
                    self.adjustify_marker(self.markers[test_idx], overlap)

                    try_again = True
                    break
            
            if try_again :
                break

        if try_again :
            
            iterations += 1

            if iterations == max_iterations :
                return False

            return self.__reposition_markers(iterations)

        return True
    
    # ##########################################################

    def adjustify_marker (self, marker, overlap) :

        extra = random.randint(5, 10)
        marker['anchor_height'] += (overlap + extra)

        # FIX ME:
        
        # now check to see if we're going to bleed
        # or sit on a crease
        
        if marker['anchor_pos'] == 'left' :
            pass
            
        elif marker['anchor_pos'] == 'top' :
            pass
            
        elif marker['anchor_pos'] == 'right' :
            pass
            
        else :
            pass
                        
    # ##########################################################
    
    def does_marker_overlap_marker(self, current, other) :

        # FIX ME:
        #
        # do I spill off the current section?
        #
        # do I cover a 'joiner' ?
        #
        # how do I know either of these things...
        
        cur_label = current['label']
        other_label = other['label']
        
        # first, ensure that some part of 'current'
        # overlaps 'other' on the x axis

        cur_x1 = current['canvas'][0]
        cur_x2 = current['canvas'][2]
        other_x1 = other['canvas'][0]
        other_x2 = other['canvas'][2]

        # print "\t%s (X) : %s, %s" % (cur_label, cur_x1, cur_x2)
        # print "\t%s (X) : %s, %s" % (other_label, other_x1, other_x2)

        if other_x1 > cur_x2 :
            return 0
        
        if other_x2 < cur_x1 :
            return 0

        cur_y = current['y']
        other_y = other['y']

        cur_cy1 = current['canvas'][1]
        cur_cy2 = current['canvas'][3]

        other_cy1 = other['canvas'][1]
        other_cy2 = other['canvas'][3]

        # do both markers occupy the same space?

        if cur_cy1 == other_cy1 and other_cy2 == cur_cy2 :
            return cur_cy2 - cur_cy1
        
        # print "%s\t%s\t%s\t%s" % (cur_cy1, cur_cy2, cur_y, cur_label)
        # print "%s\t%s\t%s\t%s" % (other_cy1, other_cy2, other_y, other_label)

        # is the y (lat) position of 'other' somewhere
        # in the space between the y position of 'current'
        # and the bottom of its pinwin canvas?
        
        if cur_cy2 >= other_y :            
            if cur_cy1 <= other_cy2 :
                return other_cy2 - cur_cy1

        # the y (or lat) position for 'other' falls between
        # the canvas of 'current'
        
        if other_y > cur_cy1 :
            if other_cy2 >= cur_cy1 :
                return other_cy2 - cur_cy1

        # ensure at least a certain amount of
        # space between markers
        
        space = cur_cy1 - other_cy2
        
        if space <= 5 :
            return random.randint(5, 10)
        
        return 0

    # ##########################################################
