__package__    = "modestMMarkers/feeds/mmGeoRSS/__init__.py"
__version__    = "0.1"
__author__     = "Aaron Straup Cope"
__url__        = "http://www.aaronland.info/python/modestMMarkers"
__date__       = "$Date: 2009/04/24 16:08:49 $"
__copyright__  = "Copyright (c) 2009 Aaron Straup Cope. BSD license : http://www.modestmaps.com/license.txt"

from modestMMarkers.feeds import mmProvider

import re
from BeautifulSoup import BeautifulStoneSoup
        
class mmGeoRSS (mmProvider) :

    def __init__ (self, *args) :
        mmProvider.__init__(self, *args)
        self.feed_type = "georss"

    #############################################################

    def parse_feed (self, data) :

        feed = {
            'title':None,
            'link':None,
            'points':[],            
            'items':{},
            'lookup':{}
            }

        p = re.compile("tag as ([^<]+)")
        soup = BeautifulStoneSoup(data)

        t = soup.find('title')

        if t :
            feed['title'] = t.contents[0]
            
        for i in soup.findAll('item'):

            # hack
            
            t = i.find('title')
            if t and t.contents[0] == 'Placemark 80' :
                continue
            
            # /hack
            
            # the basics
            
            pt = self.find_latlon(i)
    
            if not pt :
                continue

            # uid

            uid = str(i.find('guid').contents[0])
            
            # title

            t = i.find('title')

            if t :
                title = t.contents[0]
                title = self.decode_htmlentities(title)

            # description
            
            d = i.find('description')

            if d :
                desc = self.clean_description(d.string)

                tags = re.findall(p, desc)
            
                if len(tags) :
                    tags = tags[0].split(" ")
                
                    if "skip" in tags :
                        pass
                        # print "skip %s" % i                        
                        # continue

            # put it all together
            
            item = {
                'title':title,
                # label is the old...
                'label':title,                
                'latitude':pt[0],
                'longitude':pt[1],
                'tags':tags,
                'description': desc
                }

            feed['items'][uid] = item

            # lookup tables

            coord = "%s,%s" % (pt[0], pt[1])

            feed['lookup'][coord] = uid
            feed['points'].append(pt)
            
        return feed

    #############################################################
    
    def find_latlon (self, item) :

        geo = item.find('georss:point')

        if geo :
            pt = geo.contents[0]
            pt = map(lambda x: float(x), pt.strip().split(" "))

            if pt[0] != 0.0 and pt[1] != 0.0 :
                return (pt[0], pt[1])

        return None

    #############################################################    
