__package__    = "modestMMarkers/modestMMarkers/text/__init__.py"
__version__    = "0.1"
__author__     = "Aaron Straup Cope"
__url__        = "http://www.aaronland.info/python/modestMMarkers"
__date__       = "$Date: 2009/04/25 21:22:12 $"
__copyright__  = "Copyright (c) 2007-2008 Aaron Straup Cope. BSD license : http://www.modestmaps.com/license.txt"


import re
import cairo
from modestMMarkers import utils

# ##########################################################

def measure_text (ctx, txt):

        xbearing, ybearing, width, height, xadvance, yadvance = (ctx.text_extents(unicode(txt)))

        # honestly, I have no idea why I need to do this
        # but it seems necessary for long pieces of text
        # ...
        
        if width == 0.0 or width < len(txt) :
                ext = ctx.text_extents(unicode(txt))
                width = ext[2]
                height = ext[3]

        return (width, height)

# ##########################################################

def draw_text_block_to_PIL (text, width, height, **kwargs) :
        cairo_surface = draw_text_block(text, width, height, **kwargs)
        return utils.cairo2pil(cairo_surface)

# ##########################################################

def draw_text_block_to_PNG (text, path, width, height, **kwargs) :
        cairo_surface = draw_text_block(text, width, height, **kwargs)
        cairo_surface.write_to_png(path)
        
# ##########################################################

def assign_font_details (ctx, **kwargs) :
        
        font_face = "Helvetica"
        font_size = 12;

        if kwargs.has_key('font_face') :
                font_face = kwargs['font_face']

        if kwargs.has_key('font_size') :
                font_size = kwargs['font_size']

        ctx.select_font_face(font_face, cairo.FONT_SLANT_NORMAL, cairo.FONT_WEIGHT_BOLD)
        ctx.set_font_size(font_size)

# ##########################################################

def draw_text_block (text, width, height, **kwargs) :

        surface = cairo.ImageSurface(cairo.FORMAT_ARGB32, width, height)
        ctx = cairo.Context(surface)

        #

        assign_font_details(ctx, **kwargs)
        
        #
        
        if kwargs.has_key('ensure_width') :
                (w, h) = measure_text(ctx, text)

                if w > width :
                        width = int(w * 1.25)
                        
        #
                        
        chunks = []

        ok_width = width
        ok_height = height
        
        margin_x = 0
        margin_y = 0

        padding_x = 0
        padding_y = 0

        if kwargs.has_key('margin') and int(kwargs['margin']) > 0:
                ok_width -= (kwargs['margin'] * 2)
                ok_height -= (kwargs['margin'] * 2)
                
                margin_x += kwargs['margin']
                margin_y += kwargs['margin']
                                
        if kwargs.has_key('padding_left') and int(kwargs['padding_left']) > 0:
                ok_width -= kwargs['padding_left']
                margin_x = kwargs['padding_left']

        if kwargs.has_key('padding_right') and int(kwargs['padding_right']) > 0:
                ok_width -= kwargs['padding_right']

        if kwargs.has_key('padding_top') and int(kwargs['padding_top']) > 0:
                ok_height -= kwargs['padding_top']
                margin_y = kwargs['padding_top']

        if kwargs.has_key('padding_bottom') and int(kwargs['padding_bottom']) > 0:
                ok_height -= kwargs['padding_bottom']
        
        # hack, clean up variable names...
        desc = text

        ln_height = 0

        while len(desc) :
                
                (desc_width, desc_height) = measure_text(ctx, desc)

                height += desc_height

                ln_height = max(ln_height, desc_height)
                        
                if desc_width < ok_width :
                        chunks.append(desc)
                        break

                buffer = ''

                for i in range(0, len(desc)) :
                        
                        buffer += desc[i]
                        
                        (fw, fh) = measure_text(ctx, buffer)

                        ln_height = max(ln_height, fh)
                        
                        if fw < ok_width :
                                continue
                        else :

                                snippet = buffer[:-1]

                                match = re.findall("(\s([^\s]+))$", snippet)

                                if len(match) :
                                        rest = match[0][0]
                                        l = len(rest)
                                        snippet = snippet[:-l]
                    
                                        chunks.append(snippet)
                                        desc = rest.strip() + desc[i:]
                
                                        break

        width = int(width)
        height = int(height)
    
        surface = cairo.ImageSurface(cairo.FORMAT_ARGB32, width, height)
        ctx = cairo.Context(surface)

        ctx.move_to(0, 0)
        ctx.rectangle(0, 0, width, height)
        ctx.set_source_rgb(255, 255, 255)
        ctx.fill()
        
        ctx.set_source_rgb(0, 0, 0)
    
        x = margin_x + padding_x
        y = margin_y + padding_y + ln_height
                        
        ctx.move_to(x, y)

        # hrmmm....
        ln_height = 20
        
        for txt in chunks :

                txt = txt.strip()
                ok = 1
                
                for ln in txt.split("\n") :

                        if ln.strip() != '' :
                                ctx.show_text(ln)
                                
                        y = y + int(ln_height)

                        if y > ok_height :
                                ok = 0
                                break
                        else :
                                ctx.move_to(x, y)       
                if not ok :
                        break
                
        return surface

# ##########################################################

def offsets_for_page_number(ctx, pg_number) :

        # http://www.tortall.net/mu/wiki/CairoTutorial#tips-and-tricks
        
        fascent, fdescent, fheight, fxadvance, fyadvance = ctx.font_extents()
        
        xbearing, ybearing, width, height, xadvance, yadvance = (
            ctx.text_extents(unicode(pg_number)))
        
        x = 0.5 - xbearing - width / 2
        y = fdescent + fheight / 2

        # wtf do I need to adjust these again by hand?
        
        x = x - 1                
        y = y - int(height * 1.5)

        if int(pg_number) >= 10 :
                x = x - 2
                
    	return (x, y)
