__package__    = "modestMMarkers/modestMMarkers/utils/__init__.py"
__version__    = "0.1"
__author__     = "Aaron Straup Cope"
__url__        = "http://www.aaronland.info/python/modestMMarkers"
__date__       = "$Date: 2009/04/25 21:22:12 $"
__copyright__  = "Copyright (c) 2007-2008 Aaron Straup Cope. BSD license : http://www.modestmaps.com/license.txt"

import cairo
import PIL.Image

import math
import array
import re
import types

import ModestMaps
import modestMMarkers.text

# ##########################################################

def mk_points_listify (points) :

        if type(points) != types.ListType :
                return None
        
        list = []
        
        for pt in points:

                if type(pt) == types.DictType :
                        if pt.has_key('lat'):
                                list.append((pt['lat'], pt['lon']))
                        elif pt.has_key('latitude') :
                                list.append((pt['latitude'], pt['longitude']))                        
                        else :
                                pass
                else :
                        list.append(pt)

        return list

# ##########################################################

def mk_points_dictify (points, shortform=False) :

        if type(points) != types.ListType :
                return None
        
        list = []

        for pt in points :
                        
                if type(pt) == types.DictType :
                        if pt.has_key('lat'):
                                if shortform :
                                        list.append(pt)
                                else :
                                        list.append({'latitude':pt['lat'], 'longitude':pt['lon']})
                        elif pt.has_key('latitude') :
                                if shortform :
                                        list.append({'lat':pt['latitude'], 'lon':pt['longitude']})
                                else :
                                        list.append(pt)
                        else :
                                pass
                else :
                        if shortform :
                                list.append({'lat':pt[0],'lon':pt[1]})
                        else :
                                list.append({'latitude':pt[0],'longitude':pt[1]})
        return list

# ##########################################################

def latlon_to_point (mm_obj, lat, lon) :
        loc = ModestMaps.Geo.Location(lat, lon)
        return mm_obj.locationPoint(loc)

# ##########################################################

def calculate_bbox_for_points (points) :

        sw_lat = None
        sw_lon = None
        ne_lat = None
        ne_lon = None

        for pt in points :

                # ugh...
                
                if type(pt) == types.DictType :
                        if pt.has_key('lat') :
                                lat = pt['lat']
                                lon = pt['lon']                                
                        else :
                                lat = pt['latitude']
                                lon = pt['longitude']

                elif type(pt) == types.ListType :
                        lat = pt[0]
                        lon = pt[1]
                elif type(pt) == types.TupleType :
                        lat = pt[0]
                        lon = pt[1]                        
                else :
                        continue

                #
                
                if not sw_lat or lat < sw_lat :
                        sw_lat = lat
                
                if not sw_lon or lon < sw_lon :
                        sw_lon = lon
                        
                if not ne_lat or lat > ne_lat :
                        ne_lat = lat

                if not ne_lon or lon > ne_lon :
                        ne_lon = lon

        return (sw_lat,sw_lon,ne_lat,ne_lon)

# ##########################################################

def expand_bbox (bbox, adjust) :

        swlat = bbox[0]
        swlon = bbox[1]
        nelat = bbox[2]
        nelon = bbox[3]

        adjust_ne = 1
        adjust_sw = 1

        if nelon < 0 :
                adjust_ne = - adjust_ne
                
        if swlon < 0 :
                adjust_sw = - adjust_sw
                
        dist_n = __dist(nelat, nelon, nelat, (nelon + adjust_ne))
        dist_s = __dist(swlat, swlon, swlat, (swlon + adjust_sw))

        swlat = swlat - (float(adjust) / float(dist_s))
        swlon = swlon - (float(adjust) / float(dist_s))
        nelat = nelat + (float(adjust) / float(dist_n))
        nelon = nelon + (float(adjust) / float(dist_n))
        
        return [swlat, swlon, nelat, nelon]

# ##########################################################

def __dist(lat1, lon1, lat2, lon2) :
        
        theta = lon1 - lon2
        dist = math.sin(math.radians(lat1)) * math.sin(math.radians(lat2)) +  math.cos(math.radians(lat1)) * math.cos(math.radians(lat2)) * math.cos(math.radians(theta));
        dist = math.acos(dist);
        dist = math.degrees(dist);
        return dist * 60

# ##########################################################

def sort_points_by_neighbourhood (points) :
        pass

# ##########################################################

def pil2cairo (img) :

        # check me...
        img = img.convert('RGBA')
        
        (w, h) = img.size

	mode = cairo.FORMAT_ARGB32

        data = img.tostring()
        a = array.array('B', data)

        return cairo.ImageSurface.create_for_data (a, mode, w, h, (w * 4))

# ##########################################################
    
def cairo2pil(surface) :

	mode='RGBA'
        
        width = surface.get_width()
        height = surface.get_height()
    
        return PIL.Image.frombuffer(mode, (width, height), surface.get_data(), "raw", mode, 0, 1)

# ##########################################################
